<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) 2019 Amasty (https://www.amasty.com)
 * @package Amasty_Xnotif
 */
namespace Amasty\Xnotif\Helper;

use Amasty\Xnotif\Model\Source\Group;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Framework\App\Action\Action;
use Magento\ProductAlert\Block\Product\View;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\CatalogInventory\Api\Data\StockItemInterface;
use Magento\Store\Model\ScopeInterface;

class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Magento\Framework\Registry
     */
    private $registry;

    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    private $messageManager;
    
    /**
     * @var \Magento\Customer\Model\Session
     */
    private $customerSession;
    
    /**
     * @var \Magento\ProductAlert\Helper\Data
     */
    private $helper;

    /**
     * @var \Magento\Cron\Model\ResourceModel\Schedule\CollectionFactory
     */
    private $collectionFactory;

    /**
     * @var \Magento\Framework\View\Element\BlockFactory
     */
    private $blockFactory;

    /**
     * @var \Magento\Customer\Model\SessionFactory
     */
    private $sessionFactory;

    /**
     * @var \Magento\CatalogInventory\Api\StockRegistryInterface
     */
    private $stockRegistry;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;

    public function __construct(
        \Magento\Framework\Registry $registry,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\ProductAlert\Helper\Data $helper,
        \Magento\Cron\Model\ResourceModel\Schedule\CollectionFactory $collectionFactory,
        \Magento\Framework\View\Element\BlockFactory $blockFactory,
        \Magento\Customer\Model\SessionFactory $sessionFactory,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    ) {
        parent::__construct($context);
        $this->registry = $registry;
        $this->messageManager = $messageManager;
        $this->helper = $helper;
        $this->collectionFactory = $collectionFactory;
        $this->blockFactory = $blockFactory;
        $this->sessionFactory = $sessionFactory;
        $this->customerSession = $this->sessionFactory->create();
        $this->stockRegistry = $stockRegistry;
        $this->storeManager = $storeManager;
    }

    public function observeStockAlertBlock(ProductInterface $product, View $alertBlock)
    {
        $html = '';
        $currentProduct = $this->registry->registry('current_product');
        if (!$product->getId() || !$currentProduct) {
            return $html;
        }

        /*check if it is child product for replace product registered to child product.*/
        $isChildProduct = ($currentProduct->getId() != $product->getId());
        if ($isChildProduct) {
            $alertBlock->setData('parent_product_id', $currentProduct->getId());
            $alertBlock->setOriginalProduct($product);
        }
        $alertBlock->setSignupUrl($this->getSignupUrl(
            'stock',
            $product->getId(),
            $alertBlock->getData('parent_product_id')
        ));

        if ($alertBlock && !$product->getData('amxnotif_hide_alert')) {
            if (!$this->isLoggedIn()) {
                $alertBlock->setTemplate('Amasty_Xnotif::product/view_email.phtml');
            }

            $alertBlock->setData('amxnotif_observer_triggered', 1);
            $html = $alertBlock->toHtml();
            $alertBlock->setData('amxnotif_observer_triggered', null);
        }

        return $html;
    }

    public function getStockAlert(ProductInterface $product)
    {
        if (!$product || !$product->getId() || !$this->allowForCurrentCustomerGroup('stock')) {
            return '';
        }

        $alertBlock =  $this->blockFactory->createBlock(
            'Magento\ProductAlert\Block\Product\View',
            [
                'name' => 'productalert.stock.' . $product->getId()
            ]
        );

        $alertBlock->setTemplate('Magento_ProductAlert::product/view.phtml');
        $alertBlock->setHtmlClass('alert stock link-stock-alert');
        $alertBlock->setSignupLabel(__('Sign up to get notified when this configuration is back in stock'));

        return $this->observeStockAlertBlock($product, $alertBlock);
    }

    public function getPriceAlert(ProductInterface $product)
    {
        $html = '';
        $configurableProduct = $this->registry->registry('current_product');
        if ($configurableProduct) {
            $this->registry->unregister('current_product');
            $this->registry->register('current_product', $product);

            $alertBlock = $this->blockFactory->createBlock(
                'Magento\ProductAlert\Block\Product\View',
                [
                    'name' => 'productalert.price.' . $product->getId()
                ]
            );

            if ($alertBlock) {
                $alertBlock->setData('parent_product_id', $configurableProduct->getId());
                $alertBlock->setOriginalProduct($product);
                $alertBlock->setTemplate('Magento_ProductAlert::product/view.phtml');
                $alertBlock->setSignupLabel(__('Notify me when the price drops'));
            }

            $this->registry->unregister('current_product');
            $this->registry->register('current_product', $configurableProduct);
            $html = $this->observePriceAlertBlock($product, $alertBlock);
        }

        return $html;
    }
    
    public function observePriceAlertBlock(ProductInterface $product, View $alertBlock)
    {
        if (!$product->getId()) {
            return '';
        }

        if ($alertBlock && !$this->isLoggedIn()) {
            /*set template with email input*/
            $alertBlock->setTemplate('Amasty_Xnotif::product/price/view_email.phtml');
            $alertBlock->setOriginalProduct($product);
        }
        $alertBlock->setSignupUrl($this->getSignupUrl(
            'price',
            $product->getId(),
            $alertBlock->getData('parent_product_id')
        ));

        $alertBlock->setData('amxnotif_observer_triggered', 1);
        $html = $alertBlock->toHtml();
        $alertBlock->setData('amxnotif_observer_triggered', null);

        return $html;
    }

    public function getProduct()
    {
        return $this->registry->registry('product');
    }

    public function getOriginalProduct($block)
    {
        $product = $block->getOriginalProduct();
        if (!$product) {
            $product = $this->getProduct();
        }

        return $product;
    }

    /**
     * @param $type
     * @param int $productId
     *
     * @param null|int $parentId
     *
     * @return string
     */
    public function getSignupUrl($type, $productId, $parentId = null)
    {
        $params = [
            'product_id' => $productId,
            Action::PARAM_NAME_URL_ENCODED => $this->getEncodedUrl()
        ];
        if ($parentId) {
            $params['parent_id'] = $parentId;
        }

        return $this->_getUrl('xnotif/email/' . $type, $params);
    }

    /**
     * @param $type
     * @return string
     */
    public function getEmailUrl($type)
    {
        return $this->_getUrl('xnotif/email/' . $type);
    }

    public function addMessage()
    {
        $scheduleCollection = $this->collectionFactory->create()
            ->addFieldToFilter('job_code', ['eq' => 'amasty_catalog_product_alert']);
        $scheduleCollection->getSelect()->order("schedule_id desc");

        if ($scheduleCollection->getSize() == 0) {
            $this->messageManager->addNoticeMessage(
                __('No cron job "amasty_catalog_product_alert" found. Please check your cron configuration.')
            );
        }
    }

    /**
     * @param $route
     * @param array $params
     * @return string
     */
    public function getUrl($route, $params = [])
    {
        return parent::_getUrl($route, $params);
    }

    /**
     * @param $type
     * @return bool
     */
    public function allowForCurrentCustomerGroup($type)
    {
        $allowedGroups = $this->scopeConfig->getValue('amxnotif/' . $type . '/customer_group');
        $allowedGroups = str_replace(' ', '', $allowedGroups);
        $allowedGroups = explode(',', $allowedGroups);

        if (in_array(Group::ALL_GROUPS, $allowedGroups)) {
            return true;
        }

        return in_array($this->sessionFactory->create()->getCustomerGroupId(), $allowedGroups);
    }

    /**
     * @param $item
     * @param $websiteId
     *
     * @return bool
     */
    public function isItemSalable($item, $websiteId = null)
    {
        if ($websiteId === null) {
            $websiteId = $this->storeManager->getWebsite()->getId();
        }
        /** @var \Magento\CatalogInventory\Api\Data\StockStatusInterface $stockStatus */
        $stockStatus = $this->stockRegistry->getStockStatusBySku($item->getSku(), $websiteId);
        $stockStatus->setWebsiteId($websiteId);
        $saleable = $stockStatus->getStockStatus() && $this->verifyStock($stockStatus);

        return $saleable;
    }

    /**
     * @param \Magento\CatalogInventory\Api\Data\StockStatusInterface $stockStatus
     * @return bool
     */
    public function verifyStock($stockStatus)
    {
        $result = true;

        $stockItem = $stockStatus->getStockItem();

        if ($stockItem->getManageStock()) {
            if ($stockStatus->getQty() === null) {
                $result = false;
            }

            if ($stockItem->getBackorders() == StockItemInterface::BACKORDERS_NO
                && $stockStatus->getQty() <= $stockStatus->getMinQty()
            ) {
                $result = false;
            }
        }

        return $result;
    }

    /**
     * @param $item
     * @return null|string|string[]
     */
    public function getGroupedAlert($item)
    {
        $html = $this->getStockAlert($item);

        //remove form tag from content
        $html = preg_replace("/<\\/?" . 'form' . "(.|\\s)*?>/", '', $html);

        return $html;
    }

    /**
     * @param \Magento\Catalog\Model\Product $product
     * @param $websiteId
     *
     * @return int
     */
    public function getProductQty(\Magento\Catalog\Model\Product $product, $websiteId)
    {
        $stockStatus = $this->stockRegistry->getStockStatusBySku($product->getSku(), $websiteId);
        $quantity = $stockStatus->getQty();

        return $quantity;
    }

    /**
     * @return bool
     */
    public function enableQtyLimit()
    {
        return (bool)$this->getModuleConfig('stock/email_limit');
    }

    /**
     * @param $path
     * @param int $storeId
     * @return mixed
     */
    public function getModuleConfig($path, $storeId = null)
    {
        return $this->scopeConfig->getValue(
            'amxnotif/' . $path,
            ScopeInterface::SCOPE_STORE,
            $storeId
        );
    }

    /**
     * @return bool
     */
    public function isGDRPEnabled()
    {
        return (bool)$this->getModuleConfig('gdrp/enabled');
    }

    /**
     * @return string
     */
    public function getGDRPText()
    {
        return $this->getModuleConfig('gdrp/text');
    }

    /**
     * @return bool
     */
    public function isAdminNotificationEnabled()
    {
        return (bool)$this->getModuleConfig('admin_notifications/notify_admin');
    }

    /**
     * @return string
     */
    public function getPlaceholder()
    {
        return (string)$this->getModuleConfig('stock/placeholder');
    }

    /**
     * @param string $url
     * @return string
     */
    public function getEncodedUrl($url = null)
    {
        if (!$url) {
            $url = $this->_urlBuilder->getCurrentUrl();
        }
        return $this->urlEncoder->encode($url);
    }

    /**
     * @return bool
     */
    public function isLoggedIn()
    {
        $customerSession = $this->sessionFactory->create();

        return $customerSession->getCustomerId()
            && $customerSession->checkCustomerId($customerSession->getCustomerId());
    }
}
